/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.addConnectionPoint;

import org.fortiss.tooling.base.model.base.EntryConnectorBase;
import org.fortiss.tooling.base.model.base.ExitConnectorBase;
import org.fortiss.tooling.base.model.base.HierarchicElementBase;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.fortiss.tooling.common.ui.javafx.lwfxef.controller.base.DelegatingContentAnchorageController;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.fortiss.tooling.common.ui.javafx.lwfxef.mvc.IMVCBundle;

/**
 * {@link DelegatingContentAnchorageController} for diagrams with curved links.
 * 
 * @author munaro
 */
public class CurvedLinkLayoutedContentAnchorangeController<T extends IConnector & ILayoutedModelElement>
		extends LayoutedContentAnchorageController<T> {

	/** Constructor. */
	public CurvedLinkLayoutedContentAnchorangeController(IContentAnchorageMVCBundle mvcb,
			Class<T> modelType) {
		super(mvcb, modelType);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	protected void link(IMVCBundle startBundle, DiagramCoordinate startLocation,
			IMVCBundle endBundle, DiagramCoordinate endLocation) {
		// Disable direct links between hierarchic element bases
		if(startBundle.getModel() instanceof HierarchicElementBase ||
				endBundle.getModel() instanceof HierarchicElementBase) {
			return;
		}

		// Create the connection
		super.link(startBundle, startLocation, endBundle, endLocation);

		// Fetch newly created connection by comparing incoming and outgoing connections of the
		// given connectors. As only one connection is allowed between a pair of connectors, this
		// 'findFirst' is safe.
		T startConnector = (T)(startBundle.getModel() instanceof ExitConnectorBase
				? startBundle.getModel() : endBundle.getModel());
		T endConnector = (T)(endBundle.getModel() instanceof EntryConnectorBase
				? endBundle.getModel() : startBundle.getModel());
		IConnection connection = startConnector.getOutgoing().stream()
				.filter(segment -> endConnector.getIncoming().contains(segment)).findFirst().get();

		// Create handles
		DiagramCoordinate startPosition =
				new DiagramCoordinate(startBundle.getVisual().getCurrentBounds().getMaxX(),
						startBundle.getVisual().getCurrentBounds().getMaxY());
		DiagramCoordinate endPosition =
				new DiagramCoordinate(endBundle.getVisual().getCurrentBounds().getMaxX(),
						endBundle.getVisual().getCurrentBounds().getMaxY());
		DiagramCoordinate middle =
				new DiagramCoordinate((startPosition.getX() + endPosition.getX()) / 2,
						(startPosition.getY() + endPosition.getY()) / 2);
		addConnectionPoint((ILayoutedModelElement)connection, 0, (int)middle.getX(),
				(int)middle.getY());
		addConnectionPoint((ILayoutedModelElement)connection, 1, (int)middle.getX(),
				(int)middle.getY());
	}
}
